# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from osmosis.superfluid import query_pb2 as osmosis_dot_superfluid_dot_query__pb2


class QueryStub(object):
    """Query defines the gRPC querier service.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Params = channel.unary_unary(
                '/osmosis.superfluid.Query/Params',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.QueryParamsRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.QueryParamsResponse.FromString,
                )
        self.AssetType = channel.unary_unary(
                '/osmosis.superfluid.Query/AssetType',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.AssetTypeRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.AssetTypeResponse.FromString,
                )
        self.AllAssets = channel.unary_unary(
                '/osmosis.superfluid.Query/AllAssets',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.AllAssetsRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.AllAssetsResponse.FromString,
                )
        self.AssetMultiplier = channel.unary_unary(
                '/osmosis.superfluid.Query/AssetMultiplier',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.AssetMultiplierRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.AssetMultiplierResponse.FromString,
                )
        self.AllIntermediaryAccounts = channel.unary_unary(
                '/osmosis.superfluid.Query/AllIntermediaryAccounts',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.AllIntermediaryAccountsRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.AllIntermediaryAccountsResponse.FromString,
                )
        self.ConnectedIntermediaryAccount = channel.unary_unary(
                '/osmosis.superfluid.Query/ConnectedIntermediaryAccount',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.ConnectedIntermediaryAccountRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.ConnectedIntermediaryAccountResponse.FromString,
                )
        self.TotalDelegationByValidatorForDenom = channel.unary_unary(
                '/osmosis.superfluid.Query/TotalDelegationByValidatorForDenom',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByValidatorForDenomRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByValidatorForDenomResponse.FromString,
                )
        self.TotalSuperfluidDelegations = channel.unary_unary(
                '/osmosis.superfluid.Query/TotalSuperfluidDelegations',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.TotalSuperfluidDelegationsRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.TotalSuperfluidDelegationsResponse.FromString,
                )
        self.SuperfluidDelegationAmount = channel.unary_unary(
                '/osmosis.superfluid.Query/SuperfluidDelegationAmount',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationAmountRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationAmountResponse.FromString,
                )
        self.SuperfluidDelegationsByDelegator = channel.unary_unary(
                '/osmosis.superfluid.Query/SuperfluidDelegationsByDelegator',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByDelegatorRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByDelegatorResponse.FromString,
                )
        self.SuperfluidUndelegationsByDelegator = channel.unary_unary(
                '/osmosis.superfluid.Query/SuperfluidUndelegationsByDelegator',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidUndelegationsByDelegatorRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidUndelegationsByDelegatorResponse.FromString,
                )
        self.SuperfluidDelegationsByValidatorDenom = channel.unary_unary(
                '/osmosis.superfluid.Query/SuperfluidDelegationsByValidatorDenom',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByValidatorDenomRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByValidatorDenomResponse.FromString,
                )
        self.EstimateSuperfluidDelegatedAmountByValidatorDenom = channel.unary_unary(
                '/osmosis.superfluid.Query/EstimateSuperfluidDelegatedAmountByValidatorDenom',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.EstimateSuperfluidDelegatedAmountByValidatorDenomRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.EstimateSuperfluidDelegatedAmountByValidatorDenomResponse.FromString,
                )
        self.TotalDelegationByDelegator = channel.unary_unary(
                '/osmosis.superfluid.Query/TotalDelegationByDelegator',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByDelegatorRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByDelegatorResponse.FromString,
                )
        self.UnpoolWhitelist = channel.unary_unary(
                '/osmosis.superfluid.Query/UnpoolWhitelist',
                request_serializer=osmosis_dot_superfluid_dot_query__pb2.QueryUnpoolWhitelistRequest.SerializeToString,
                response_deserializer=osmosis_dot_superfluid_dot_query__pb2.QueryUnpoolWhitelistResponse.FromString,
                )


class QueryServicer(object):
    """Query defines the gRPC querier service.
    """

    def Params(self, request, context):
        """Params returns the total set of superfluid parameters.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AssetType(self, request, context):
        """Returns superfluid asset type, whether if it's a native asset or an lp
        share.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AllAssets(self, request, context):
        """Returns all registered superfluid assets.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AssetMultiplier(self, request, context):
        """Returns the osmo equivalent multiplier used in the most recent epoch.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AllIntermediaryAccounts(self, request, context):
        """Returns all superfluid intermediary accounts.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ConnectedIntermediaryAccount(self, request, context):
        """Returns intermediary account connected to a superfluid staked lock by id
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TotalDelegationByValidatorForDenom(self, request, context):
        """Returns the amount of delegations of specific denom for all validators
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TotalSuperfluidDelegations(self, request, context):
        """Returns the total amount of osmo superfluidly staked.
        Response is denominated in uosmo.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SuperfluidDelegationAmount(self, request, context):
        """Returns the coins superfluid delegated for the delegator, validator, denom
        triplet
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SuperfluidDelegationsByDelegator(self, request, context):
        """Returns all the delegated superfluid poistions for a specific delegator.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SuperfluidUndelegationsByDelegator(self, request, context):
        """Returns all the undelegating superfluid poistions for a specific delegator.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SuperfluidDelegationsByValidatorDenom(self, request, context):
        """Returns all the superfluid positions of a specific denom delegated to one
        validator
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def EstimateSuperfluidDelegatedAmountByValidatorDenom(self, request, context):
        """Returns the amount of a specific denom delegated to a specific validator
        This is labeled an estimate, because the way it calculates the amount can
        lead rounding errors from the true delegated amount
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TotalDelegationByDelegator(self, request, context):
        """Returns the specified delegations for a specific delegator
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UnpoolWhitelist(self, request, context):
        """Returns a list of whitelisted pool ids to unpool.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QueryServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Params': grpc.unary_unary_rpc_method_handler(
                    servicer.Params,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.QueryParamsRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.QueryParamsResponse.SerializeToString,
            ),
            'AssetType': grpc.unary_unary_rpc_method_handler(
                    servicer.AssetType,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.AssetTypeRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.AssetTypeResponse.SerializeToString,
            ),
            'AllAssets': grpc.unary_unary_rpc_method_handler(
                    servicer.AllAssets,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.AllAssetsRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.AllAssetsResponse.SerializeToString,
            ),
            'AssetMultiplier': grpc.unary_unary_rpc_method_handler(
                    servicer.AssetMultiplier,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.AssetMultiplierRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.AssetMultiplierResponse.SerializeToString,
            ),
            'AllIntermediaryAccounts': grpc.unary_unary_rpc_method_handler(
                    servicer.AllIntermediaryAccounts,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.AllIntermediaryAccountsRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.AllIntermediaryAccountsResponse.SerializeToString,
            ),
            'ConnectedIntermediaryAccount': grpc.unary_unary_rpc_method_handler(
                    servicer.ConnectedIntermediaryAccount,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.ConnectedIntermediaryAccountRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.ConnectedIntermediaryAccountResponse.SerializeToString,
            ),
            'TotalDelegationByValidatorForDenom': grpc.unary_unary_rpc_method_handler(
                    servicer.TotalDelegationByValidatorForDenom,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByValidatorForDenomRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByValidatorForDenomResponse.SerializeToString,
            ),
            'TotalSuperfluidDelegations': grpc.unary_unary_rpc_method_handler(
                    servicer.TotalSuperfluidDelegations,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.TotalSuperfluidDelegationsRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.TotalSuperfluidDelegationsResponse.SerializeToString,
            ),
            'SuperfluidDelegationAmount': grpc.unary_unary_rpc_method_handler(
                    servicer.SuperfluidDelegationAmount,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationAmountRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationAmountResponse.SerializeToString,
            ),
            'SuperfluidDelegationsByDelegator': grpc.unary_unary_rpc_method_handler(
                    servicer.SuperfluidDelegationsByDelegator,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByDelegatorRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByDelegatorResponse.SerializeToString,
            ),
            'SuperfluidUndelegationsByDelegator': grpc.unary_unary_rpc_method_handler(
                    servicer.SuperfluidUndelegationsByDelegator,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidUndelegationsByDelegatorRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidUndelegationsByDelegatorResponse.SerializeToString,
            ),
            'SuperfluidDelegationsByValidatorDenom': grpc.unary_unary_rpc_method_handler(
                    servicer.SuperfluidDelegationsByValidatorDenom,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByValidatorDenomRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByValidatorDenomResponse.SerializeToString,
            ),
            'EstimateSuperfluidDelegatedAmountByValidatorDenom': grpc.unary_unary_rpc_method_handler(
                    servicer.EstimateSuperfluidDelegatedAmountByValidatorDenom,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.EstimateSuperfluidDelegatedAmountByValidatorDenomRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.EstimateSuperfluidDelegatedAmountByValidatorDenomResponse.SerializeToString,
            ),
            'TotalDelegationByDelegator': grpc.unary_unary_rpc_method_handler(
                    servicer.TotalDelegationByDelegator,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByDelegatorRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByDelegatorResponse.SerializeToString,
            ),
            'UnpoolWhitelist': grpc.unary_unary_rpc_method_handler(
                    servicer.UnpoolWhitelist,
                    request_deserializer=osmosis_dot_superfluid_dot_query__pb2.QueryUnpoolWhitelistRequest.FromString,
                    response_serializer=osmosis_dot_superfluid_dot_query__pb2.QueryUnpoolWhitelistResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'osmosis.superfluid.Query', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Query(object):
    """Query defines the gRPC querier service.
    """

    @staticmethod
    def Params(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/Params',
            osmosis_dot_superfluid_dot_query__pb2.QueryParamsRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.QueryParamsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AssetType(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/AssetType',
            osmosis_dot_superfluid_dot_query__pb2.AssetTypeRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.AssetTypeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AllAssets(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/AllAssets',
            osmosis_dot_superfluid_dot_query__pb2.AllAssetsRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.AllAssetsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AssetMultiplier(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/AssetMultiplier',
            osmosis_dot_superfluid_dot_query__pb2.AssetMultiplierRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.AssetMultiplierResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AllIntermediaryAccounts(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/AllIntermediaryAccounts',
            osmosis_dot_superfluid_dot_query__pb2.AllIntermediaryAccountsRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.AllIntermediaryAccountsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ConnectedIntermediaryAccount(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/ConnectedIntermediaryAccount',
            osmosis_dot_superfluid_dot_query__pb2.ConnectedIntermediaryAccountRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.ConnectedIntermediaryAccountResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TotalDelegationByValidatorForDenom(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/TotalDelegationByValidatorForDenom',
            osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByValidatorForDenomRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByValidatorForDenomResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TotalSuperfluidDelegations(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/TotalSuperfluidDelegations',
            osmosis_dot_superfluid_dot_query__pb2.TotalSuperfluidDelegationsRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.TotalSuperfluidDelegationsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SuperfluidDelegationAmount(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/SuperfluidDelegationAmount',
            osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationAmountRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationAmountResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SuperfluidDelegationsByDelegator(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/SuperfluidDelegationsByDelegator',
            osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByDelegatorRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByDelegatorResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SuperfluidUndelegationsByDelegator(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/SuperfluidUndelegationsByDelegator',
            osmosis_dot_superfluid_dot_query__pb2.SuperfluidUndelegationsByDelegatorRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.SuperfluidUndelegationsByDelegatorResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SuperfluidDelegationsByValidatorDenom(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/SuperfluidDelegationsByValidatorDenom',
            osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByValidatorDenomRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.SuperfluidDelegationsByValidatorDenomResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def EstimateSuperfluidDelegatedAmountByValidatorDenom(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/EstimateSuperfluidDelegatedAmountByValidatorDenom',
            osmosis_dot_superfluid_dot_query__pb2.EstimateSuperfluidDelegatedAmountByValidatorDenomRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.EstimateSuperfluidDelegatedAmountByValidatorDenomResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TotalDelegationByDelegator(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/TotalDelegationByDelegator',
            osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByDelegatorRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.QueryTotalDelegationByDelegatorResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UnpoolWhitelist(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/osmosis.superfluid.Query/UnpoolWhitelist',
            osmosis_dot_superfluid_dot_query__pb2.QueryUnpoolWhitelistRequest.SerializeToString,
            osmosis_dot_superfluid_dot_query__pb2.QueryUnpoolWhitelistResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import osmosis.tokenfactory.v1beta1.tx_pb2

class MsgStub:
    """Msg defines the tokefactory module's gRPC message service."""

    def __init__(self, channel: grpc.Channel) -> None: ...
    CreateDenom: grpc.UnaryUnaryMultiCallable[
        osmosis.tokenfactory.v1beta1.tx_pb2.MsgCreateDenom,
        osmosis.tokenfactory.v1beta1.tx_pb2.MsgCreateDenomResponse,
    ]
    Mint: grpc.UnaryUnaryMultiCallable[
        osmosis.tokenfactory.v1beta1.tx_pb2.MsgMint,
        osmosis.tokenfactory.v1beta1.tx_pb2.MsgMintResponse,
    ]
    Burn: grpc.UnaryUnaryMultiCallable[
        osmosis.tokenfactory.v1beta1.tx_pb2.MsgBurn,
        osmosis.tokenfactory.v1beta1.tx_pb2.MsgBurnResponse,
    ]
    ChangeAdmin: grpc.UnaryUnaryMultiCallable[
        osmosis.tokenfactory.v1beta1.tx_pb2.MsgChangeAdmin,
        osmosis.tokenfactory.v1beta1.tx_pb2.MsgChangeAdminResponse,
    ]
    SetDenomMetadata: grpc.UnaryUnaryMultiCallable[
        osmosis.tokenfactory.v1beta1.tx_pb2.MsgSetDenomMetadata,
        osmosis.tokenfactory.v1beta1.tx_pb2.MsgSetDenomMetadataResponse,
    ]

class MsgServicer(metaclass=abc.ABCMeta):
    """Msg defines the tokefactory module's gRPC message service."""

    @abc.abstractmethod
    def CreateDenom(
        self,
        request: osmosis.tokenfactory.v1beta1.tx_pb2.MsgCreateDenom,
        context: grpc.ServicerContext,
    ) -> osmosis.tokenfactory.v1beta1.tx_pb2.MsgCreateDenomResponse: ...
    @abc.abstractmethod
    def Mint(
        self,
        request: osmosis.tokenfactory.v1beta1.tx_pb2.MsgMint,
        context: grpc.ServicerContext,
    ) -> osmosis.tokenfactory.v1beta1.tx_pb2.MsgMintResponse: ...
    @abc.abstractmethod
    def Burn(
        self,
        request: osmosis.tokenfactory.v1beta1.tx_pb2.MsgBurn,
        context: grpc.ServicerContext,
    ) -> osmosis.tokenfactory.v1beta1.tx_pb2.MsgBurnResponse: ...
    @abc.abstractmethod
    def ChangeAdmin(
        self,
        request: osmosis.tokenfactory.v1beta1.tx_pb2.MsgChangeAdmin,
        context: grpc.ServicerContext,
    ) -> osmosis.tokenfactory.v1beta1.tx_pb2.MsgChangeAdminResponse: ...
    @abc.abstractmethod
    def SetDenomMetadata(
        self,
        request: osmosis.tokenfactory.v1beta1.tx_pb2.MsgSetDenomMetadata,
        context: grpc.ServicerContext,
    ) -> osmosis.tokenfactory.v1beta1.tx_pb2.MsgSetDenomMetadataResponse: ...

def add_MsgServicer_to_server(servicer: MsgServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import cosmos.base.query.v1beta1.pagination_pb2
import cosmos.base.v1beta1.coin_pb2
import cosmos.distribution.v1beta1.distribution_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class QueryParamsRequest(google.protobuf.message.Message):
    """QueryParamsRequest is the request type for the Query/Params RPC method."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___QueryParamsRequest = QueryParamsRequest

@typing_extensions.final
class QueryParamsResponse(google.protobuf.message.Message):
    """QueryParamsResponse is the response type for the Query/Params RPC method."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> cosmos.distribution.v1beta1.distribution_pb2.Params:
        """params defines the parameters of the module."""
    def __init__(
        self,
        *,
        params: cosmos.distribution.v1beta1.distribution_pb2.Params | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params", b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["params", b"params"]) -> None: ...

global___QueryParamsResponse = QueryParamsResponse

@typing_extensions.final
class QueryValidatorOutstandingRewardsRequest(google.protobuf.message.Message):
    """QueryValidatorOutstandingRewardsRequest is the request type for the
    Query/ValidatorOutstandingRewards RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    validator_address: builtins.str
    """validator_address defines the validator address to query for."""
    def __init__(
        self,
        *,
        validator_address: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator_address", b"validator_address"]) -> None: ...

global___QueryValidatorOutstandingRewardsRequest = QueryValidatorOutstandingRewardsRequest

@typing_extensions.final
class QueryValidatorOutstandingRewardsResponse(google.protobuf.message.Message):
    """QueryValidatorOutstandingRewardsResponse is the response type for the
    Query/ValidatorOutstandingRewards RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REWARDS_FIELD_NUMBER: builtins.int
    @property
    def rewards(self) -> cosmos.distribution.v1beta1.distribution_pb2.ValidatorOutstandingRewards: ...
    def __init__(
        self,
        *,
        rewards: cosmos.distribution.v1beta1.distribution_pb2.ValidatorOutstandingRewards | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["rewards", b"rewards"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["rewards", b"rewards"]) -> None: ...

global___QueryValidatorOutstandingRewardsResponse = QueryValidatorOutstandingRewardsResponse

@typing_extensions.final
class QueryValidatorCommissionRequest(google.protobuf.message.Message):
    """QueryValidatorCommissionRequest is the request type for the
    Query/ValidatorCommission RPC method
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    validator_address: builtins.str
    """validator_address defines the validator address to query for."""
    def __init__(
        self,
        *,
        validator_address: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator_address", b"validator_address"]) -> None: ...

global___QueryValidatorCommissionRequest = QueryValidatorCommissionRequest

@typing_extensions.final
class QueryValidatorCommissionResponse(google.protobuf.message.Message):
    """QueryValidatorCommissionResponse is the response type for the
    Query/ValidatorCommission RPC method
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMISSION_FIELD_NUMBER: builtins.int
    @property
    def commission(self) -> cosmos.distribution.v1beta1.distribution_pb2.ValidatorAccumulatedCommission:
        """commission defines the commision the validator received."""
    def __init__(
        self,
        *,
        commission: cosmos.distribution.v1beta1.distribution_pb2.ValidatorAccumulatedCommission | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["commission", b"commission"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commission", b"commission"]) -> None: ...

global___QueryValidatorCommissionResponse = QueryValidatorCommissionResponse

@typing_extensions.final
class QueryValidatorSlashesRequest(google.protobuf.message.Message):
    """QueryValidatorSlashesRequest is the request type for the
    Query/ValidatorSlashes RPC method
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    STARTING_HEIGHT_FIELD_NUMBER: builtins.int
    ENDING_HEIGHT_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    validator_address: builtins.str
    """validator_address defines the validator address to query for."""
    starting_height: builtins.int
    """starting_height defines the optional starting height to query the slashes."""
    ending_height: builtins.int
    """starting_height defines the optional ending height to query the slashes."""
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
    def __init__(
        self,
        *,
        validator_address: builtins.str = ...,
        starting_height: builtins.int = ...,
        ending_height: builtins.int = ...,
        pagination: cosmos.base.query.v1beta1.pagination_pb2.PageRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ending_height", b"ending_height", "pagination", b"pagination", "starting_height", b"starting_height", "validator_address", b"validator_address"]) -> None: ...

global___QueryValidatorSlashesRequest = QueryValidatorSlashesRequest

@typing_extensions.final
class QueryValidatorSlashesResponse(google.protobuf.message.Message):
    """QueryValidatorSlashesResponse is the response type for the
    Query/ValidatorSlashes RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SLASHES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def slashes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.distribution.v1beta1.distribution_pb2.ValidatorSlashEvent]:
        """slashes defines the slashes the validator received."""
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
    def __init__(
        self,
        *,
        slashes: collections.abc.Iterable[cosmos.distribution.v1beta1.distribution_pb2.ValidatorSlashEvent] | None = ...,
        pagination: cosmos.base.query.v1beta1.pagination_pb2.PageResponse | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "slashes", b"slashes"]) -> None: ...

global___QueryValidatorSlashesResponse = QueryValidatorSlashesResponse

@typing_extensions.final
class QueryDelegationRewardsRequest(google.protobuf.message.Message):
    """QueryDelegationRewardsRequest is the request type for the
    Query/DelegationRewards RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: builtins.str
    """delegator_address defines the delegator address to query for."""
    validator_address: builtins.str
    """validator_address defines the validator address to query for."""
    def __init__(
        self,
        *,
        delegator_address: builtins.str = ...,
        validator_address: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address", b"delegator_address", "validator_address", b"validator_address"]) -> None: ...

global___QueryDelegationRewardsRequest = QueryDelegationRewardsRequest

@typing_extensions.final
class QueryDelegationRewardsResponse(google.protobuf.message.Message):
    """QueryDelegationRewardsResponse is the response type for the
    Query/DelegationRewards RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REWARDS_FIELD_NUMBER: builtins.int
    @property
    def rewards(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]:
        """rewards defines the rewards accrued by a delegation."""
    def __init__(
        self,
        *,
        rewards: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["rewards", b"rewards"]) -> None: ...

global___QueryDelegationRewardsResponse = QueryDelegationRewardsResponse

@typing_extensions.final
class QueryDelegationTotalRewardsRequest(google.protobuf.message.Message):
    """QueryDelegationTotalRewardsRequest is the request type for the
    Query/DelegationTotalRewards RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: builtins.str
    """delegator_address defines the delegator address to query for."""
    def __init__(
        self,
        *,
        delegator_address: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address", b"delegator_address"]) -> None: ...

global___QueryDelegationTotalRewardsRequest = QueryDelegationTotalRewardsRequest

@typing_extensions.final
class QueryDelegationTotalRewardsResponse(google.protobuf.message.Message):
    """QueryDelegationTotalRewardsResponse is the response type for the
    Query/DelegationTotalRewards RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REWARDS_FIELD_NUMBER: builtins.int
    TOTAL_FIELD_NUMBER: builtins.int
    @property
    def rewards(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.distribution.v1beta1.distribution_pb2.DelegationDelegatorReward]:
        """rewards defines all the rewards accrued by a delegator."""
    @property
    def total(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]:
        """total defines the sum of all the rewards."""
    def __init__(
        self,
        *,
        rewards: collections.abc.Iterable[cosmos.distribution.v1beta1.distribution_pb2.DelegationDelegatorReward] | None = ...,
        total: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["rewards", b"rewards", "total", b"total"]) -> None: ...

global___QueryDelegationTotalRewardsResponse = QueryDelegationTotalRewardsResponse

@typing_extensions.final
class QueryDelegatorValidatorsRequest(google.protobuf.message.Message):
    """QueryDelegatorValidatorsRequest is the request type for the
    Query/DelegatorValidators RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: builtins.str
    """delegator_address defines the delegator address to query for."""
    def __init__(
        self,
        *,
        delegator_address: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address", b"delegator_address"]) -> None: ...

global___QueryDelegatorValidatorsRequest = QueryDelegatorValidatorsRequest

@typing_extensions.final
class QueryDelegatorValidatorsResponse(google.protobuf.message.Message):
    """QueryDelegatorValidatorsResponse is the response type for the
    Query/DelegatorValidators RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALIDATORS_FIELD_NUMBER: builtins.int
    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """validators defines the validators a delegator is delegating for."""
    def __init__(
        self,
        *,
        validators: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validators", b"validators"]) -> None: ...

global___QueryDelegatorValidatorsResponse = QueryDelegatorValidatorsResponse

@typing_extensions.final
class QueryDelegatorWithdrawAddressRequest(google.protobuf.message.Message):
    """QueryDelegatorWithdrawAddressRequest is the request type for the
    Query/DelegatorWithdrawAddress RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: builtins.str
    """delegator_address defines the delegator address to query for."""
    def __init__(
        self,
        *,
        delegator_address: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address", b"delegator_address"]) -> None: ...

global___QueryDelegatorWithdrawAddressRequest = QueryDelegatorWithdrawAddressRequest

@typing_extensions.final
class QueryDelegatorWithdrawAddressResponse(google.protobuf.message.Message):
    """QueryDelegatorWithdrawAddressResponse is the response type for the
    Query/DelegatorWithdrawAddress RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WITHDRAW_ADDRESS_FIELD_NUMBER: builtins.int
    withdraw_address: builtins.str
    """withdraw_address defines the delegator address to query for."""
    def __init__(
        self,
        *,
        withdraw_address: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["withdraw_address", b"withdraw_address"]) -> None: ...

global___QueryDelegatorWithdrawAddressResponse = QueryDelegatorWithdrawAddressResponse

@typing_extensions.final
class QueryCommunityPoolRequest(google.protobuf.message.Message):
    """QueryCommunityPoolRequest is the request type for the Query/CommunityPool RPC
    method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___QueryCommunityPoolRequest = QueryCommunityPoolRequest

@typing_extensions.final
class QueryCommunityPoolResponse(google.protobuf.message.Message):
    """QueryCommunityPoolResponse is the response type for the Query/CommunityPool
    RPC method.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POOL_FIELD_NUMBER: builtins.int
    @property
    def pool(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]:
        """pool defines community pool's coins."""
    def __init__(
        self,
        *,
        pool: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool", b"pool"]) -> None: ...

global___QueryCommunityPoolResponse = QueryCommunityPoolResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _LockQueryType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _LockQueryTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_LockQueryType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ByDuration: _LockQueryType.ValueType  # 0
    ByTime: _LockQueryType.ValueType  # 1

class LockQueryType(_LockQueryType, metaclass=_LockQueryTypeEnumTypeWrapper):
    """LockQueryType defines the type of the lock query that can
    either be by duration or start time of the lock.
    """

ByDuration: LockQueryType.ValueType  # 0
ByTime: LockQueryType.ValueType  # 1
global___LockQueryType = LockQueryType

@typing_extensions.final
class PeriodLock(google.protobuf.message.Message):
    """PeriodLock is a single lock unit by period defined by the x/lockup module.
    It's a record of a locked coin at a specific time. It stores owner, duration,
    unlock time and the number of coins locked. A state of a period lock is
    created upon lock creation, and deleted once the lock has been matured after
    the `duration` has passed since unbonding started.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    END_TIME_FIELD_NUMBER: builtins.int
    COINS_FIELD_NUMBER: builtins.int
    ID: builtins.int
    """ID is the unique id of the lock.
    The ID of the lock is decided upon lock creation, incrementing by 1 for
    every lock.
    """
    owner: builtins.str
    """Owner is the account address of the lock owner.
    Only the owner can modify the state of the lock.
    """
    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration:
        """Duration is the time needed for a lock to mature after unlocking has
        started.
        """
    @property
    def end_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """EndTime refers to the time at which the lock would mature and get deleted.
        This value is first initialized when an unlock has started for the lock,
        end time being block time + duration.
        """
    @property
    def coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """Coins are the tokens locked within the lock, kept in the module account."""
    def __init__(
        self,
        *,
        ID: builtins.int = ...,
        owner: builtins.str = ...,
        duration: google.protobuf.duration_pb2.Duration | None = ...,
        end_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        coins: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["duration", b"duration", "end_time", b"end_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ID", b"ID", "coins", b"coins", "duration", b"duration", "end_time", b"end_time", "owner", b"owner"]) -> None: ...

global___PeriodLock = PeriodLock

@typing_extensions.final
class QueryCondition(google.protobuf.message.Message):
    """QueryCondition is a struct used for querying locks upon different conditions.
    Duration field and timestamp fields could be optional, depending on the
    LockQueryType.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOCK_QUERY_TYPE_FIELD_NUMBER: builtins.int
    DENOM_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    lock_query_type: global___LockQueryType.ValueType
    """LockQueryType is a type of lock query, ByLockDuration | ByLockTime"""
    denom: builtins.str
    """Denom represents the token denomination we are looking to lock up"""
    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration:
        """Duration is used to query locks with longer duration than the specified
        duration. Duration field must not be nil when the lock query type is
        `ByLockDuration`.
        """
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Timestamp is used by locks started before the specified duration.
        Timestamp field must not be nil when the lock query type is `ByLockTime`.
        Querying locks with timestamp is currently not implemented.
        """
    def __init__(
        self,
        *,
        lock_query_type: global___LockQueryType.ValueType = ...,
        denom: builtins.str = ...,
        duration: google.protobuf.duration_pb2.Duration | None = ...,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["duration", b"duration", "timestamp", b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["denom", b"denom", "duration", b"duration", "lock_query_type", b"lock_query_type", "timestamp", b"timestamp"]) -> None: ...

global___QueryCondition = QueryCondition

@typing_extensions.final
class SyntheticLock(google.protobuf.message.Message):
    """SyntheticLock is creating virtual lockup where new denom is combination of
    original denom and synthetic suffix. At the time of synthetic lockup creation
    and deletion, accumulation store is also being updated and on querier side,
    they can query as freely as native lockup.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNDERLYING_LOCK_ID_FIELD_NUMBER: builtins.int
    SYNTH_DENOM_FIELD_NUMBER: builtins.int
    END_TIME_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    underlying_lock_id: builtins.int
    """Underlying Lock ID is the underlying native lock's id for this synthetic
    lockup. A synthetic lock MUST have an underlying lock.
    """
    synth_denom: builtins.str
    """SynthDenom is the synthetic denom that is a combination of
    gamm share + bonding status + validator address.
    """
    @property
    def end_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """used for unbonding synthetic lockups, for active synthetic lockups, this
        value is set to uninitialized value
        """
    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration:
        """Duration is the duration for a synthetic lock to mature
        at the point of unbonding has started.
        """
    def __init__(
        self,
        *,
        underlying_lock_id: builtins.int = ...,
        synth_denom: builtins.str = ...,
        end_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        duration: google.protobuf.duration_pb2.Duration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["duration", b"duration", "end_time", b"end_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["duration", b"duration", "end_time", b"end_time", "synth_denom", b"synth_denom", "underlying_lock_id", b"underlying_lock_id"]) -> None: ...

global___SyntheticLock = SyntheticLock



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.slashing.v1beta1.query_pb2
import grpc

class QueryStub:
    """Query provides defines the gRPC querier service"""

    def __init__(self, channel: grpc.Channel) -> None: ...
    Params: grpc.UnaryUnaryMultiCallable[
        cosmos.slashing.v1beta1.query_pb2.QueryParamsRequest,
        cosmos.slashing.v1beta1.query_pb2.QueryParamsResponse,
    ]
    """Params queries the parameters of slashing module"""
    SigningInfo: grpc.UnaryUnaryMultiCallable[
        cosmos.slashing.v1beta1.query_pb2.QuerySigningInfoRequest,
        cosmos.slashing.v1beta1.query_pb2.QuerySigningInfoResponse,
    ]
    """SigningInfo queries the signing info of given cons address"""
    SigningInfos: grpc.UnaryUnaryMultiCallable[
        cosmos.slashing.v1beta1.query_pb2.QuerySigningInfosRequest,
        cosmos.slashing.v1beta1.query_pb2.QuerySigningInfosResponse,
    ]
    """SigningInfos queries signing info of all validators"""

class QueryServicer(metaclass=abc.ABCMeta):
    """Query provides defines the gRPC querier service"""

    @abc.abstractmethod
    def Params(
        self,
        request: cosmos.slashing.v1beta1.query_pb2.QueryParamsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.slashing.v1beta1.query_pb2.QueryParamsResponse:
        """Params queries the parameters of slashing module"""
    @abc.abstractmethod
    def SigningInfo(
        self,
        request: cosmos.slashing.v1beta1.query_pb2.QuerySigningInfoRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.slashing.v1beta1.query_pb2.QuerySigningInfoResponse:
        """SigningInfo queries the signing info of given cons address"""
    @abc.abstractmethod
    def SigningInfos(
        self,
        request: cosmos.slashing.v1beta1.query_pb2.QuerySigningInfosRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.slashing.v1beta1.query_pb2.QuerySigningInfosResponse:
        """SigningInfos queries signing info of all validators"""

def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/twap/v1beta1/query.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from osmosis.twap.v1beta1 import twap_record_pb2 as osmosis_dot_twap_dot_v1beta1_dot_twap__record__pb2
from osmosis.twap.v1beta1 import genesis_pb2 as osmosis_dot_twap_dot_v1beta1_dot_genesis__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from cosmos.base.query.v1beta1 import pagination_pb2 as cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2
from google_apis.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n osmosis/twap/v1beta1/query.proto\x12\x14osmosis.twap.v1beta1\x1a\x14gogoproto/gogo.proto\x1a&osmosis/twap/v1beta1/twap_record.proto\x1a\"osmosis/twap/v1beta1/genesis.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a*cosmos/base/query/v1beta1/pagination.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x19google/protobuf/any.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xeb\x01\n\x15\x41rithmeticTwapRequest\x12\x0f\n\x07pool_id\x18\x01 \x01(\x04\x12\x12\n\nbase_asset\x18\x02 \x01(\t\x12\x13\n\x0bquote_asset\x18\x03 \x01(\t\x12M\n\nstart_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x1d\xc8\xde\x1f\x00\x90\xdf\x1f\x01\xf2\xde\x1f\x11yaml:\"start_time\"\x12I\n\x08\x65nd_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x1b\xc8\xde\x1f\x01\x90\xdf\x1f\x01\xf2\xde\x1f\x0fyaml:\"end_time\"\"{\n\x16\x41rithmeticTwapResponse\x12\x61\n\x0f\x61rithmetic_twap\x18\x01 \x01(\tBH\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xf2\xde\x1f\x16yaml:\"arithmetic_twap\"\xc8\xde\x1f\x00\"\xa5\x01\n\x1a\x41rithmeticTwapToNowRequest\x12\x0f\n\x07pool_id\x18\x01 \x01(\x04\x12\x12\n\nbase_asset\x18\x02 \x01(\t\x12\x13\n\x0bquote_asset\x18\x03 \x01(\t\x12M\n\nstart_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x1d\xc8\xde\x1f\x00\x90\xdf\x1f\x01\xf2\xde\x1f\x11yaml:\"start_time\"\"\x80\x01\n\x1b\x41rithmeticTwapToNowResponse\x12\x61\n\x0f\x61rithmetic_twap\x18\x01 \x01(\tBH\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xf2\xde\x1f\x16yaml:\"arithmetic_twap\"\xc8\xde\x1f\x00\"\xea\x01\n\x14GeometricTwapRequest\x12\x0f\n\x07pool_id\x18\x01 \x01(\x04\x12\x12\n\nbase_asset\x18\x02 \x01(\t\x12\x13\n\x0bquote_asset\x18\x03 \x01(\t\x12M\n\nstart_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x1d\xc8\xde\x1f\x00\x90\xdf\x1f\x01\xf2\xde\x1f\x11yaml:\"start_time\"\x12I\n\x08\x65nd_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x1b\xc8\xde\x1f\x01\x90\xdf\x1f\x01\xf2\xde\x1f\x0fyaml:\"end_time\"\"x\n\x15GeometricTwapResponse\x12_\n\x0egeometric_twap\x18\x01 \x01(\tBG\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xf2\xde\x1f\x15yaml:\"geometric_twap\"\xc8\xde\x1f\x00\"\xa4\x01\n\x19GeometricTwapToNowRequest\x12\x0f\n\x07pool_id\x18\x01 \x01(\x04\x12\x12\n\nbase_asset\x18\x02 \x01(\t\x12\x13\n\x0bquote_asset\x18\x03 \x01(\t\x12M\n\nstart_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x1d\xc8\xde\x1f\x00\x90\xdf\x1f\x01\xf2\xde\x1f\x11yaml:\"start_time\"\"}\n\x1aGeometricTwapToNowResponse\x12_\n\x0egeometric_twap\x18\x01 \x01(\tBG\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xf2\xde\x1f\x15yaml:\"geometric_twap\"\xc8\xde\x1f\x00\"\x0f\n\rParamsRequest\"D\n\x0eParamsResponse\x12\x32\n\x06params\x18\x01 \x01(\x0b\x32\x1c.osmosis.twap.v1beta1.ParamsB\x04\xc8\xde\x1f\x00\x32\x92\x06\n\x05Query\x12y\n\x06Params\x12#.osmosis.twap.v1beta1.ParamsRequest\x1a$.osmosis.twap.v1beta1.ParamsResponse\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/osmosis/twap/v1beta1/Params\x12\x99\x01\n\x0e\x41rithmeticTwap\x12+.osmosis.twap.v1beta1.ArithmeticTwapRequest\x1a,.osmosis.twap.v1beta1.ArithmeticTwapResponse\",\x82\xd3\xe4\x93\x02&\x12$/osmosis/twap/v1beta1/ArithmeticTwap\x12\xad\x01\n\x13\x41rithmeticTwapToNow\x12\x30.osmosis.twap.v1beta1.ArithmeticTwapToNowRequest\x1a\x31.osmosis.twap.v1beta1.ArithmeticTwapToNowResponse\"1\x82\xd3\xe4\x93\x02+\x12)/osmosis/twap/v1beta1/ArithmeticTwapToNow\x12\x95\x01\n\rGeometricTwap\x12*.osmosis.twap.v1beta1.GeometricTwapRequest\x1a+.osmosis.twap.v1beta1.GeometricTwapResponse\"+\x82\xd3\xe4\x93\x02%\x12#/osmosis/twap/v1beta1/GeometricTwap\x12\xa9\x01\n\x12GeometricTwapToNow\x12/.osmosis.twap.v1beta1.GeometricTwapToNowRequest\x1a\x30.osmosis.twap.v1beta1.GeometricTwapToNowResponse\"0\x82\xd3\xe4\x93\x02*\x12(/osmosis/twap/v1beta1/GeometricTwapToNowB>Z<github.com/osmosis-labs/osmosis/v14/x/twap/client/queryprotob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.twap.v1beta1.query_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z<github.com/osmosis-labs/osmosis/v14/x/twap/client/queryproto'
  _ARITHMETICTWAPREQUEST.fields_by_name['start_time']._options = None
  _ARITHMETICTWAPREQUEST.fields_by_name['start_time']._serialized_options = b'\310\336\037\000\220\337\037\001\362\336\037\021yaml:\"start_time\"'
  _ARITHMETICTWAPREQUEST.fields_by_name['end_time']._options = None
  _ARITHMETICTWAPREQUEST.fields_by_name['end_time']._serialized_options = b'\310\336\037\001\220\337\037\001\362\336\037\017yaml:\"end_time\"'
  _ARITHMETICTWAPRESPONSE.fields_by_name['arithmetic_twap']._options = None
  _ARITHMETICTWAPRESPONSE.fields_by_name['arithmetic_twap']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\362\336\037\026yaml:\"arithmetic_twap\"\310\336\037\000'
  _ARITHMETICTWAPTONOWREQUEST.fields_by_name['start_time']._options = None
  _ARITHMETICTWAPTONOWREQUEST.fields_by_name['start_time']._serialized_options = b'\310\336\037\000\220\337\037\001\362\336\037\021yaml:\"start_time\"'
  _ARITHMETICTWAPTONOWRESPONSE.fields_by_name['arithmetic_twap']._options = None
  _ARITHMETICTWAPTONOWRESPONSE.fields_by_name['arithmetic_twap']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\362\336\037\026yaml:\"arithmetic_twap\"\310\336\037\000'
  _GEOMETRICTWAPREQUEST.fields_by_name['start_time']._options = None
  _GEOMETRICTWAPREQUEST.fields_by_name['start_time']._serialized_options = b'\310\336\037\000\220\337\037\001\362\336\037\021yaml:\"start_time\"'
  _GEOMETRICTWAPREQUEST.fields_by_name['end_time']._options = None
  _GEOMETRICTWAPREQUEST.fields_by_name['end_time']._serialized_options = b'\310\336\037\001\220\337\037\001\362\336\037\017yaml:\"end_time\"'
  _GEOMETRICTWAPRESPONSE.fields_by_name['geometric_twap']._options = None
  _GEOMETRICTWAPRESPONSE.fields_by_name['geometric_twap']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\362\336\037\025yaml:\"geometric_twap\"\310\336\037\000'
  _GEOMETRICTWAPTONOWREQUEST.fields_by_name['start_time']._options = None
  _GEOMETRICTWAPTONOWREQUEST.fields_by_name['start_time']._serialized_options = b'\310\336\037\000\220\337\037\001\362\336\037\021yaml:\"start_time\"'
  _GEOMETRICTWAPTONOWRESPONSE.fields_by_name['geometric_twap']._options = None
  _GEOMETRICTWAPTONOWRESPONSE.fields_by_name['geometric_twap']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\362\336\037\025yaml:\"geometric_twap\"\310\336\037\000'
  _PARAMSRESPONSE.fields_by_name['params']._options = None
  _PARAMSRESPONSE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _QUERY.methods_by_name['Params']._options = None
  _QUERY.methods_by_name['Params']._serialized_options = b'\202\323\344\223\002\036\022\034/osmosis/twap/v1beta1/Params'
  _QUERY.methods_by_name['ArithmeticTwap']._options = None
  _QUERY.methods_by_name['ArithmeticTwap']._serialized_options = b'\202\323\344\223\002&\022$/osmosis/twap/v1beta1/ArithmeticTwap'
  _QUERY.methods_by_name['ArithmeticTwapToNow']._options = None
  _QUERY.methods_by_name['ArithmeticTwapToNow']._serialized_options = b'\202\323\344\223\002+\022)/osmosis/twap/v1beta1/ArithmeticTwapToNow'
  _QUERY.methods_by_name['GeometricTwap']._options = None
  _QUERY.methods_by_name['GeometricTwap']._serialized_options = b'\202\323\344\223\002%\022#/osmosis/twap/v1beta1/GeometricTwap'
  _QUERY.methods_by_name['GeometricTwapToNow']._options = None
  _QUERY.methods_by_name['GeometricTwapToNow']._serialized_options = b'\202\323\344\223\002*\022(/osmosis/twap/v1beta1/GeometricTwapToNow'
  _ARITHMETICTWAPREQUEST._serialized_start=350
  _ARITHMETICTWAPREQUEST._serialized_end=585
  _ARITHMETICTWAPRESPONSE._serialized_start=587
  _ARITHMETICTWAPRESPONSE._serialized_end=710
  _ARITHMETICTWAPTONOWREQUEST._serialized_start=713
  _ARITHMETICTWAPTONOWREQUEST._serialized_end=878
  _ARITHMETICTWAPTONOWRESPONSE._serialized_start=881
  _ARITHMETICTWAPTONOWRESPONSE._serialized_end=1009
  _GEOMETRICTWAPREQUEST._serialized_start=1012
  _GEOMETRICTWAPREQUEST._serialized_end=1246
  _GEOMETRICTWAPRESPONSE._serialized_start=1248
  _GEOMETRICTWAPRESPONSE._serialized_end=1368
  _GEOMETRICTWAPTONOWREQUEST._serialized_start=1371
  _GEOMETRICTWAPTONOWREQUEST._serialized_end=1535
  _GEOMETRICTWAPTONOWRESPONSE._serialized_start=1537
  _GEOMETRICTWAPTONOWRESPONSE._serialized_end=1662
  _PARAMSREQUEST._serialized_start=1664
  _PARAMSREQUEST._serialized_end=1679
  _PARAMSRESPONSE._serialized_start=1681
  _PARAMSRESPONSE._serialized_end=1749
  _QUERY._serialized_start=1752
  _QUERY._serialized_end=2538
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import cosmos.base.v1beta1.coin_pb2
import cosmos.vesting.v1beta1.vesting_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class MsgCreateVestingAccount(google.protobuf.message.Message):
    """MsgCreateVestingAccount defines a message that enables creating a vesting
    account.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FROM_ADDRESS_FIELD_NUMBER: builtins.int
    TO_ADDRESS_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    END_TIME_FIELD_NUMBER: builtins.int
    DELAYED_FIELD_NUMBER: builtins.int
    from_address: builtins.str
    to_address: builtins.str
    @property
    def amount(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    end_time: builtins.int
    delayed: builtins.bool
    def __init__(
        self,
        *,
        from_address: builtins.str = ...,
        to_address: builtins.str = ...,
        amount: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
        end_time: builtins.int = ...,
        delayed: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount", b"amount", "delayed", b"delayed", "end_time", b"end_time", "from_address", b"from_address", "to_address", b"to_address"]) -> None: ...

global___MsgCreateVestingAccount = MsgCreateVestingAccount

@typing_extensions.final
class MsgCreateVestingAccountResponse(google.protobuf.message.Message):
    """MsgCreateVestingAccountResponse defines the Msg/CreateVestingAccount response type."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___MsgCreateVestingAccountResponse = MsgCreateVestingAccountResponse

@typing_extensions.final
class MsgCreateClawbackVestingAccount(google.protobuf.message.Message):
    """MsgCreateClawbackVestingAccount defines a message that enables creating a ClawbackVestingAccount."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FROM_ADDRESS_FIELD_NUMBER: builtins.int
    TO_ADDRESS_FIELD_NUMBER: builtins.int
    START_TIME_FIELD_NUMBER: builtins.int
    LOCKUP_PERIODS_FIELD_NUMBER: builtins.int
    VESTING_PERIODS_FIELD_NUMBER: builtins.int
    MERGE_FIELD_NUMBER: builtins.int
    from_address: builtins.str
    """from_address specifies the account to provide the funds and sign the
    clawback request
    """
    to_address: builtins.str
    """to_address specifies the account to receive the funds"""
    start_time: builtins.int
    """start_time defines the time at which the vesting period begins"""
    @property
    def lockup_periods(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.vesting.v1beta1.vesting_pb2.Period]:
        """lockup_periods defines the unlocking schedule relative to the start_time"""
    @property
    def vesting_periods(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.vesting.v1beta1.vesting_pb2.Period]:
        """vesting_periods defines the vesting schedule relative to the start_time"""
    merge: builtins.bool
    """merge specifies a creation mechanism for existing
    ClawbackVestingAccounts. If true, merge this new grant into an existing
    ClawbackVestingAccount, or create it if it does not exist. If false,
    creates a new account. New grants to an existing account must be from the
    same from_address.
    """
    def __init__(
        self,
        *,
        from_address: builtins.str = ...,
        to_address: builtins.str = ...,
        start_time: builtins.int = ...,
        lockup_periods: collections.abc.Iterable[cosmos.vesting.v1beta1.vesting_pb2.Period] | None = ...,
        vesting_periods: collections.abc.Iterable[cosmos.vesting.v1beta1.vesting_pb2.Period] | None = ...,
        merge: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["from_address", b"from_address", "lockup_periods", b"lockup_periods", "merge", b"merge", "start_time", b"start_time", "to_address", b"to_address", "vesting_periods", b"vesting_periods"]) -> None: ...

global___MsgCreateClawbackVestingAccount = MsgCreateClawbackVestingAccount

@typing_extensions.final
class MsgCreateClawbackVestingAccountResponse(google.protobuf.message.Message):
    """MsgCreateClawbackVestingAccountResponse defines the
    MsgCreateClawbackVestingAccount response type.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___MsgCreateClawbackVestingAccountResponse = MsgCreateClawbackVestingAccountResponse

@typing_extensions.final
class MsgClawback(google.protobuf.message.Message):
    """MsgClawback defines a message that removes unvested tokens from a
    ClawbackVestingAccount.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FUNDER_ADDRESS_FIELD_NUMBER: builtins.int
    ADDRESS_FIELD_NUMBER: builtins.int
    DEST_ADDRESS_FIELD_NUMBER: builtins.int
    funder_address: builtins.str
    """funder_address is the address which funded the account"""
    address: builtins.str
    """address is the address of the ClawbackVestingAccount to claw back from."""
    dest_address: builtins.str
    """dest_address specifies where the clawed-back tokens should be transferred
    to. If empty, the tokens will be transferred back to the original funder of
    the account.
    """
    def __init__(
        self,
        *,
        funder_address: builtins.str = ...,
        address: builtins.str = ...,
        dest_address: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["address", b"address", "dest_address", b"dest_address", "funder_address", b"funder_address"]) -> None: ...

global___MsgClawback = MsgClawback

@typing_extensions.final
class MsgClawbackResponse(google.protobuf.message.Message):
    """MsgClawbackResponse defines the MsgClawback response type."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___MsgClawbackResponse = MsgClawbackResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Minter(google.protobuf.message.Message):
    """Minter represents the minting state."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EPOCH_PROVISIONS_FIELD_NUMBER: builtins.int
    epoch_provisions: builtins.str
    """epoch_provisions represent rewards for the current epoch."""
    def __init__(
        self,
        *,
        epoch_provisions: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["epoch_provisions", b"epoch_provisions"]) -> None: ...

global___Minter = Minter

@typing_extensions.final
class WeightedAddress(google.protobuf.message.Message):
    """WeightedAddress represents an address with a weight assigned to it.
    The weight is used to determine the proportion of the total minted
    tokens to be minted to the address.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    WEIGHT_FIELD_NUMBER: builtins.int
    address: builtins.str
    weight: builtins.str
    def __init__(
        self,
        *,
        address: builtins.str = ...,
        weight: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["address", b"address", "weight", b"weight"]) -> None: ...

global___WeightedAddress = WeightedAddress

@typing_extensions.final
class DistributionProportions(google.protobuf.message.Message):
    """DistributionProportions defines the distribution proportions of the minted
    denom. In other words, defines which stakeholders will receive the minted
    denoms and how much.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STAKING_FIELD_NUMBER: builtins.int
    POOL_INCENTIVES_FIELD_NUMBER: builtins.int
    DEVELOPER_REWARDS_FIELD_NUMBER: builtins.int
    COMMUNITY_POOL_FIELD_NUMBER: builtins.int
    staking: builtins.str
    """staking defines the proportion of the minted mint_denom that is to be
    allocated as staking rewards.
    """
    pool_incentives: builtins.str
    """pool_incentives defines the proportion of the minted mint_denom that is
    to be allocated as pool incentives.
    """
    developer_rewards: builtins.str
    """developer_rewards defines the proportion of the minted mint_denom that is
    to be allocated to developer rewards address.
    """
    community_pool: builtins.str
    """community_pool defines the proportion of the minted mint_denom that is
    to be allocated to the community pool.
    """
    def __init__(
        self,
        *,
        staking: builtins.str = ...,
        pool_incentives: builtins.str = ...,
        developer_rewards: builtins.str = ...,
        community_pool: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["community_pool", b"community_pool", "developer_rewards", b"developer_rewards", "pool_incentives", b"pool_incentives", "staking", b"staking"]) -> None: ...

global___DistributionProportions = DistributionProportions

@typing_extensions.final
class Params(google.protobuf.message.Message):
    """Params holds parameters for the x/mint module."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MINT_DENOM_FIELD_NUMBER: builtins.int
    GENESIS_EPOCH_PROVISIONS_FIELD_NUMBER: builtins.int
    EPOCH_IDENTIFIER_FIELD_NUMBER: builtins.int
    REDUCTION_PERIOD_IN_EPOCHS_FIELD_NUMBER: builtins.int
    REDUCTION_FACTOR_FIELD_NUMBER: builtins.int
    DISTRIBUTION_PROPORTIONS_FIELD_NUMBER: builtins.int
    WEIGHTED_DEVELOPER_REWARDS_RECEIVERS_FIELD_NUMBER: builtins.int
    MINTING_REWARDS_DISTRIBUTION_START_EPOCH_FIELD_NUMBER: builtins.int
    mint_denom: builtins.str
    """mint_denom is the denom of the coin to mint."""
    genesis_epoch_provisions: builtins.str
    """genesis_epoch_provisions epoch provisions from the first epoch."""
    epoch_identifier: builtins.str
    """epoch_identifier mint epoch identifier e.g. (day, week)."""
    reduction_period_in_epochs: builtins.int
    """reduction_period_in_epochs the number of epochs it takes
    to reduce the rewards.
    """
    reduction_factor: builtins.str
    """reduction_factor is the reduction multiplier to execute
    at the end of each period set by reduction_period_in_epochs.
    """
    @property
    def distribution_proportions(self) -> global___DistributionProportions:
        """distribution_proportions defines the distribution proportions of the minted
        denom. In other words, defines which stakeholders will receive the minted
        denoms and how much.
        """
    @property
    def weighted_developer_rewards_receivers(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___WeightedAddress]:
        """weighted_developer_rewards_receivers is the address to receive developer
        rewards with weights assignedt to each address. The final amount that each
        address receives is: epoch_provisions *
        distribution_proportions.developer_rewards * Address's Weight.
        """
    minting_rewards_distribution_start_epoch: builtins.int
    """minting_rewards_distribution_start_epoch start epoch to distribute minting
    rewards
    """
    def __init__(
        self,
        *,
        mint_denom: builtins.str = ...,
        genesis_epoch_provisions: builtins.str = ...,
        epoch_identifier: builtins.str = ...,
        reduction_period_in_epochs: builtins.int = ...,
        reduction_factor: builtins.str = ...,
        distribution_proportions: global___DistributionProportions | None = ...,
        weighted_developer_rewards_receivers: collections.abc.Iterable[global___WeightedAddress] | None = ...,
        minting_rewards_distribution_start_epoch: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["distribution_proportions", b"distribution_proportions"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["distribution_proportions", b"distribution_proportions", "epoch_identifier", b"epoch_identifier", "genesis_epoch_provisions", b"genesis_epoch_provisions", "mint_denom", b"mint_denom", "minting_rewards_distribution_start_epoch", b"minting_rewards_distribution_start_epoch", "reduction_factor", b"reduction_factor", "reduction_period_in_epochs", b"reduction_period_in_epochs", "weighted_developer_rewards_receivers", b"weighted_developer_rewards_receivers"]) -> None: ...

global___Params = Params



import dataclasses
from typing import Any, Dict, List, Union

from osmosispy import utils
from osmosispy.pytypes import event


@dataclasses.dataclass
class TxResp:
    """
    A 'TxResp' represents the response payload from a successful transaction.

    The 'TxResponse' type is defined in
    [cosmos-sdk/types/abci.pb.go](https://github.com/cosmos/cosmos-sdk/blob/v0.45.10/types/abci.pb.go)

    ### Args & Attributes:

    - height (int): block height at which the transaction was committed.
    - txhash (str): unique identifier for the transaction
    - data (str): Result bytes.
    - rawLog (List[TxLogEvents]): Raw output of the SDK application's logger.
        Possibly non-deterministic. This output also contains the events emitted
        during the processing of the transaction, which is equivalently
    - logs (list): Typed output of the SDK application's logger.
        Possibly non-deterministic.
    - gasWanted (str): Amount of gas units requested for the transaction.
    - gasUsed (str): Amount of gas units consumed by the transaction execution.
    - events (list): Tendermint events emitted by processing the transaction.
        The events in this attribute include those emitted by both from
        the ante handler and the processing of all messages, whereas the
        'rawLog' events are only those emitted when processing messages (with
        additional metadata).
    - _raw (RawTxResp): The unprocessed form of the transaction resposnse.

    """

    height: int
    txhash: str
    data: str
    rawLog: List[event.TxLogEvents]
    logs: list
    gasWanted: int
    gasUsed: int
    events: list
    _raw: 'RawTxResp'

    @classmethod
    def from_raw(cls, raw_tx_resp: 'RawTxResp') -> 'TxResp':
        return cls(
            height=int(raw_tx_resp["height"]),
            txhash=raw_tx_resp["txhash"],
            data=raw_tx_resp["data"],
            rawLog=[
                event.TxLogEvents(msg_log['events'])
                for msg_log in raw_tx_resp["rawLog"]
            ],
            logs=raw_tx_resp["logs"],
            gasWanted=int(raw_tx_resp["gasWanted"]),
            gasUsed=int(raw_tx_resp["gasUsed"]),
            events=raw_tx_resp["events"],
            _raw=raw_tx_resp,
        )

    def __repr__(self) -> str:
        repr_body = ", ".join(
            [
                f"height={self.height}",
                f"txhash={self.txhash}",
                f"gasUsed={self.gasUsed}",
                f"gasWanted={self.gasWanted}",
                f"rawLog={self.rawLog}",
            ]
        )
        return f"TxResp({repr_body})"


# from typing import TypedDict  # not available in Python 3.7
# class RawTxResp(TypedDict):
class RawTxResp(dict):
    """Proxy for a 'TypedDict' representing a transaction response.
    - The 'TxResponse' type is defined in
    [cosmos-sdk/types/abci.pb.go](https://github.com/cosmos/cosmos-sdk/blob/v0.45.10/types/abci.pb.go)

    ### Keys (ValueType):

    - height (str): block height at which the transaction was committed.
    - txhash (str): unique identifier for the transaction
    - data (str): Result bytes.
    - rawLog (list): Raw output of the SDK application's logger.
        Possibly non-deterministic. This output also contains the events emitted
        during the processing of the transaction, which is equivalently
    - logs (list): Typed output of the SDK application's logger.
        Possibly non-deterministic.
    - gasWanted (str): Amount of gas units requested for the transaction.
    - gasUsed (str): Amount of gas units consumed by the transaction execution.
    - events (list): Tendermint events emitted by processing the transaction.
        The events in this attribute include those emitted by both from
        the ante handler and the processing of all messages, whereas the
        'rawLog' events are only those emitted when processing messages (with
        additional metadata).
    """

    def __new__(cls, _dict: Dict[str, Any]) -> Dict[str, Union[str, list]]:
        """Verifies that the dictionary has the expected keys."""
        keys_wanted = ["height", "txhash", "data", "rawLog", "logs"] + [
            "gasWanted",
            "gasUsed",
            "events",
        ]
        utils.dict_keys_must_match(_dict, keys_wanted)
        return _dict



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Params(google.protobuf.message.Message):
    """Params defines the parameters for the tokenfactory module."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DENOM_CREATION_FEE_FIELD_NUMBER: builtins.int
    @property
    def denom_creation_fee(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(
        self,
        *,
        denom_creation_fee: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["denom_creation_fee", b"denom_creation_fee"]) -> None: ...

global___Params = Params



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/epochs/query.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google_apis.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from cosmos.base.query.v1beta1 import pagination_pb2 as cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2
from osmosis.epochs import genesis_pb2 as osmosis_dot_epochs_dot_genesis__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1aosmosis/epochs/query.proto\x12\x16osmosis.epochs.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x1cgoogle/api/annotations.proto\x1a*cosmos/base/query/v1beta1/pagination.proto\x1a\x1cosmosis/epochs/genesis.proto\"\x18\n\x16QueryEpochsInfoRequest\"R\n\x17QueryEpochsInfoResponse\x12\x37\n\x06\x65pochs\x18\x01 \x03(\x0b\x32!.osmosis.epochs.v1beta1.EpochInfoB\x04\xc8\xde\x1f\x00\".\n\x18QueryCurrentEpochRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\"2\n\x19QueryCurrentEpochResponse\x12\x15\n\rcurrent_epoch\x18\x01 \x01(\x03\x32\xc4\x02\n\x05Query\x12\x95\x01\n\nEpochInfos\x12..osmosis.epochs.v1beta1.QueryEpochsInfoRequest\x1a/.osmosis.epochs.v1beta1.QueryEpochsInfoResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/osmosis/epochs/v1beta1/epochs\x12\xa2\x01\n\x0c\x43urrentEpoch\x12\x30.osmosis.epochs.v1beta1.QueryCurrentEpochRequest\x1a\x31.osmosis.epochs.v1beta1.QueryCurrentEpochResponse\"-\x82\xd3\xe4\x93\x02\'\x12%/osmosis/epochs/v1beta1/current_epochB4Z2github.com/osmosis-labs/osmosis/v14/x/epochs/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.epochs.query_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2github.com/osmosis-labs/osmosis/v14/x/epochs/types'
  _QUERYEPOCHSINFORESPONSE.fields_by_name['epochs']._options = None
  _QUERYEPOCHSINFORESPONSE.fields_by_name['epochs']._serialized_options = b'\310\336\037\000'
  _QUERY.methods_by_name['EpochInfos']._options = None
  _QUERY.methods_by_name['EpochInfos']._serialized_options = b'\202\323\344\223\002 \022\036/osmosis/epochs/v1beta1/epochs'
  _QUERY.methods_by_name['CurrentEpoch']._options = None
  _QUERY.methods_by_name['CurrentEpoch']._serialized_options = b'\202\323\344\223\002\'\022%/osmosis/epochs/v1beta1/current_epoch'
  _QUERYEPOCHSINFOREQUEST._serialized_start=180
  _QUERYEPOCHSINFOREQUEST._serialized_end=204
  _QUERYEPOCHSINFORESPONSE._serialized_start=206
  _QUERYEPOCHSINFORESPONSE._serialized_end=288
  _QUERYCURRENTEPOCHREQUEST._serialized_start=290
  _QUERYCURRENTEPOCHREQUEST._serialized_end=336
  _QUERYCURRENTEPOCHRESPONSE._serialized_start=338
  _QUERYCURRENTEPOCHRESPONSE._serialized_end=388
  _QUERY._serialized_start=391
  _QUERY._serialized_end=715
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import cosmos.base.query.v1beta1.pagination_pb2
import cosmos.base.v1beta1.coin_pb2
import cosmos.staking.v1beta1.staking_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import osmosis.lockup.lock_pb2
import osmosis.superfluid.params_pb2
import osmosis.superfluid.superfluid_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class QueryParamsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___QueryParamsRequest = QueryParamsRequest

@typing_extensions.final
class QueryParamsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> osmosis.superfluid.params_pb2.Params:
        """params defines the parameters of the module."""
    def __init__(
        self,
        *,
        params: osmosis.superfluid.params_pb2.Params | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params", b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["params", b"params"]) -> None: ...

global___QueryParamsResponse = QueryParamsResponse

@typing_extensions.final
class AssetTypeRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DENOM_FIELD_NUMBER: builtins.int
    denom: builtins.str
    def __init__(
        self,
        *,
        denom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["denom", b"denom"]) -> None: ...

global___AssetTypeRequest = AssetTypeRequest

@typing_extensions.final
class AssetTypeResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ASSET_TYPE_FIELD_NUMBER: builtins.int
    asset_type: osmosis.superfluid.superfluid_pb2.SuperfluidAssetType.ValueType
    def __init__(
        self,
        *,
        asset_type: osmosis.superfluid.superfluid_pb2.SuperfluidAssetType.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["asset_type", b"asset_type"]) -> None: ...

global___AssetTypeResponse = AssetTypeResponse

@typing_extensions.final
class AllAssetsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___AllAssetsRequest = AllAssetsRequest

@typing_extensions.final
class AllAssetsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ASSETS_FIELD_NUMBER: builtins.int
    @property
    def assets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[osmosis.superfluid.superfluid_pb2.SuperfluidAsset]: ...
    def __init__(
        self,
        *,
        assets: collections.abc.Iterable[osmosis.superfluid.superfluid_pb2.SuperfluidAsset] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["assets", b"assets"]) -> None: ...

global___AllAssetsResponse = AllAssetsResponse

@typing_extensions.final
class AssetMultiplierRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DENOM_FIELD_NUMBER: builtins.int
    denom: builtins.str
    def __init__(
        self,
        *,
        denom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["denom", b"denom"]) -> None: ...

global___AssetMultiplierRequest = AssetMultiplierRequest

@typing_extensions.final
class AssetMultiplierResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OSMO_EQUIVALENT_MULTIPLIER_FIELD_NUMBER: builtins.int
    @property
    def osmo_equivalent_multiplier(self) -> osmosis.superfluid.superfluid_pb2.OsmoEquivalentMultiplierRecord: ...
    def __init__(
        self,
        *,
        osmo_equivalent_multiplier: osmosis.superfluid.superfluid_pb2.OsmoEquivalentMultiplierRecord | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["osmo_equivalent_multiplier", b"osmo_equivalent_multiplier"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["osmo_equivalent_multiplier", b"osmo_equivalent_multiplier"]) -> None: ...

global___AssetMultiplierResponse = AssetMultiplierResponse

@typing_extensions.final
class SuperfluidIntermediaryAccountInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DENOM_FIELD_NUMBER: builtins.int
    VAL_ADDR_FIELD_NUMBER: builtins.int
    GAUGE_ID_FIELD_NUMBER: builtins.int
    ADDRESS_FIELD_NUMBER: builtins.int
    denom: builtins.str
    val_addr: builtins.str
    gauge_id: builtins.int
    address: builtins.str
    def __init__(
        self,
        *,
        denom: builtins.str = ...,
        val_addr: builtins.str = ...,
        gauge_id: builtins.int = ...,
        address: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["address", b"address", "denom", b"denom", "gauge_id", b"gauge_id", "val_addr", b"val_addr"]) -> None: ...

global___SuperfluidIntermediaryAccountInfo = SuperfluidIntermediaryAccountInfo

@typing_extensions.final
class AllIntermediaryAccountsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...
    def __init__(
        self,
        *,
        pagination: cosmos.base.query.v1beta1.pagination_pb2.PageRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> None: ...

global___AllIntermediaryAccountsRequest = AllIntermediaryAccountsRequest

@typing_extensions.final
class AllIntermediaryAccountsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACCOUNTS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def accounts(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___SuperfluidIntermediaryAccountInfo]: ...
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...
    def __init__(
        self,
        *,
        accounts: collections.abc.Iterable[global___SuperfluidIntermediaryAccountInfo] | None = ...,
        pagination: cosmos.base.query.v1beta1.pagination_pb2.PageResponse | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["accounts", b"accounts", "pagination", b"pagination"]) -> None: ...

global___AllIntermediaryAccountsResponse = AllIntermediaryAccountsResponse

@typing_extensions.final
class ConnectedIntermediaryAccountRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOCK_ID_FIELD_NUMBER: builtins.int
    lock_id: builtins.int
    def __init__(
        self,
        *,
        lock_id: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lock_id", b"lock_id"]) -> None: ...

global___ConnectedIntermediaryAccountRequest = ConnectedIntermediaryAccountRequest

@typing_extensions.final
class ConnectedIntermediaryAccountResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACCOUNT_FIELD_NUMBER: builtins.int
    @property
    def account(self) -> global___SuperfluidIntermediaryAccountInfo: ...
    def __init__(
        self,
        *,
        account: global___SuperfluidIntermediaryAccountInfo | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["account", b"account"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["account", b"account"]) -> None: ...

global___ConnectedIntermediaryAccountResponse = ConnectedIntermediaryAccountResponse

@typing_extensions.final
class QueryTotalDelegationByValidatorForDenomRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DENOM_FIELD_NUMBER: builtins.int
    denom: builtins.str
    def __init__(
        self,
        *,
        denom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["denom", b"denom"]) -> None: ...

global___QueryTotalDelegationByValidatorForDenomRequest = QueryTotalDelegationByValidatorForDenomRequest

@typing_extensions.final
class QueryTotalDelegationByValidatorForDenomResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ASSETS_FIELD_NUMBER: builtins.int
    @property
    def assets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Delegations]: ...
    def __init__(
        self,
        *,
        assets: collections.abc.Iterable[global___Delegations] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["assets", b"assets"]) -> None: ...

global___QueryTotalDelegationByValidatorForDenomResponse = QueryTotalDelegationByValidatorForDenomResponse

@typing_extensions.final
class Delegations(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VAL_ADDR_FIELD_NUMBER: builtins.int
    AMOUNT_SFSD_FIELD_NUMBER: builtins.int
    OSMO_EQUIVALENT_FIELD_NUMBER: builtins.int
    val_addr: builtins.str
    amount_sfsd: builtins.str
    osmo_equivalent: builtins.str
    def __init__(
        self,
        *,
        val_addr: builtins.str = ...,
        amount_sfsd: builtins.str = ...,
        osmo_equivalent: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount_sfsd", b"amount_sfsd", "osmo_equivalent", b"osmo_equivalent", "val_addr", b"val_addr"]) -> None: ...

global___Delegations = Delegations

@typing_extensions.final
class TotalSuperfluidDelegationsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___TotalSuperfluidDelegationsRequest = TotalSuperfluidDelegationsRequest

@typing_extensions.final
class TotalSuperfluidDelegationsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TOTAL_DELEGATIONS_FIELD_NUMBER: builtins.int
    total_delegations: builtins.str
    def __init__(
        self,
        *,
        total_delegations: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["total_delegations", b"total_delegations"]) -> None: ...

global___TotalSuperfluidDelegationsResponse = TotalSuperfluidDelegationsResponse

@typing_extensions.final
class SuperfluidDelegationAmountRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    DENOM_FIELD_NUMBER: builtins.int
    delegator_address: builtins.str
    validator_address: builtins.str
    denom: builtins.str
    def __init__(
        self,
        *,
        delegator_address: builtins.str = ...,
        validator_address: builtins.str = ...,
        denom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address", b"delegator_address", "denom", b"denom", "validator_address", b"validator_address"]) -> None: ...

global___SuperfluidDelegationAmountRequest = SuperfluidDelegationAmountRequest

@typing_extensions.final
class SuperfluidDelegationAmountResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AMOUNT_FIELD_NUMBER: builtins.int
    @property
    def amount(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(
        self,
        *,
        amount: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount", b"amount"]) -> None: ...

global___SuperfluidDelegationAmountResponse = SuperfluidDelegationAmountResponse

@typing_extensions.final
class SuperfluidDelegationsByDelegatorRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: builtins.str
    def __init__(
        self,
        *,
        delegator_address: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address", b"delegator_address"]) -> None: ...

global___SuperfluidDelegationsByDelegatorRequest = SuperfluidDelegationsByDelegatorRequest

@typing_extensions.final
class SuperfluidDelegationsByDelegatorResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUPERFLUID_DELEGATION_RECORDS_FIELD_NUMBER: builtins.int
    TOTAL_DELEGATED_COINS_FIELD_NUMBER: builtins.int
    TOTAL_EQUIVALENT_STAKED_AMOUNT_FIELD_NUMBER: builtins.int
    @property
    def superfluid_delegation_records(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[osmosis.superfluid.superfluid_pb2.SuperfluidDelegationRecord]: ...
    @property
    def total_delegated_coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def total_equivalent_staked_amount(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(
        self,
        *,
        superfluid_delegation_records: collections.abc.Iterable[osmosis.superfluid.superfluid_pb2.SuperfluidDelegationRecord] | None = ...,
        total_delegated_coins: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
        total_equivalent_staked_amount: cosmos.base.v1beta1.coin_pb2.Coin | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["total_equivalent_staked_amount", b"total_equivalent_staked_amount"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["superfluid_delegation_records", b"superfluid_delegation_records", "total_delegated_coins", b"total_delegated_coins", "total_equivalent_staked_amount", b"total_equivalent_staked_amount"]) -> None: ...

global___SuperfluidDelegationsByDelegatorResponse = SuperfluidDelegationsByDelegatorResponse

@typing_extensions.final
class SuperfluidUndelegationsByDelegatorRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    DENOM_FIELD_NUMBER: builtins.int
    delegator_address: builtins.str
    denom: builtins.str
    def __init__(
        self,
        *,
        delegator_address: builtins.str = ...,
        denom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address", b"delegator_address", "denom", b"denom"]) -> None: ...

global___SuperfluidUndelegationsByDelegatorRequest = SuperfluidUndelegationsByDelegatorRequest

@typing_extensions.final
class SuperfluidUndelegationsByDelegatorResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUPERFLUID_DELEGATION_RECORDS_FIELD_NUMBER: builtins.int
    TOTAL_UNDELEGATED_COINS_FIELD_NUMBER: builtins.int
    SYNTHETIC_LOCKS_FIELD_NUMBER: builtins.int
    @property
    def superfluid_delegation_records(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[osmosis.superfluid.superfluid_pb2.SuperfluidDelegationRecord]: ...
    @property
    def total_undelegated_coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def synthetic_locks(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[osmosis.lockup.lock_pb2.SyntheticLock]: ...
    def __init__(
        self,
        *,
        superfluid_delegation_records: collections.abc.Iterable[osmosis.superfluid.superfluid_pb2.SuperfluidDelegationRecord] | None = ...,
        total_undelegated_coins: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
        synthetic_locks: collections.abc.Iterable[osmosis.lockup.lock_pb2.SyntheticLock] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["superfluid_delegation_records", b"superfluid_delegation_records", "synthetic_locks", b"synthetic_locks", "total_undelegated_coins", b"total_undelegated_coins"]) -> None: ...

global___SuperfluidUndelegationsByDelegatorResponse = SuperfluidUndelegationsByDelegatorResponse

@typing_extensions.final
class SuperfluidDelegationsByValidatorDenomRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    DENOM_FIELD_NUMBER: builtins.int
    validator_address: builtins.str
    denom: builtins.str
    def __init__(
        self,
        *,
        validator_address: builtins.str = ...,
        denom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["denom", b"denom", "validator_address", b"validator_address"]) -> None: ...

global___SuperfluidDelegationsByValidatorDenomRequest = SuperfluidDelegationsByValidatorDenomRequest

@typing_extensions.final
class SuperfluidDelegationsByValidatorDenomResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUPERFLUID_DELEGATION_RECORDS_FIELD_NUMBER: builtins.int
    @property
    def superfluid_delegation_records(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[osmosis.superfluid.superfluid_pb2.SuperfluidDelegationRecord]: ...
    def __init__(
        self,
        *,
        superfluid_delegation_records: collections.abc.Iterable[osmosis.superfluid.superfluid_pb2.SuperfluidDelegationRecord] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["superfluid_delegation_records", b"superfluid_delegation_records"]) -> None: ...

global___SuperfluidDelegationsByValidatorDenomResponse = SuperfluidDelegationsByValidatorDenomResponse

@typing_extensions.final
class EstimateSuperfluidDelegatedAmountByValidatorDenomRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    DENOM_FIELD_NUMBER: builtins.int
    validator_address: builtins.str
    denom: builtins.str
    def __init__(
        self,
        *,
        validator_address: builtins.str = ...,
        denom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["denom", b"denom", "validator_address", b"validator_address"]) -> None: ...

global___EstimateSuperfluidDelegatedAmountByValidatorDenomRequest = EstimateSuperfluidDelegatedAmountByValidatorDenomRequest

@typing_extensions.final
class EstimateSuperfluidDelegatedAmountByValidatorDenomResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TOTAL_DELEGATED_COINS_FIELD_NUMBER: builtins.int
    @property
    def total_delegated_coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(
        self,
        *,
        total_delegated_coins: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["total_delegated_coins", b"total_delegated_coins"]) -> None: ...

global___EstimateSuperfluidDelegatedAmountByValidatorDenomResponse = EstimateSuperfluidDelegatedAmountByValidatorDenomResponse

@typing_extensions.final
class QueryTotalDelegationByDelegatorRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: builtins.str
    def __init__(
        self,
        *,
        delegator_address: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address", b"delegator_address"]) -> None: ...

global___QueryTotalDelegationByDelegatorRequest = QueryTotalDelegationByDelegatorRequest

@typing_extensions.final
class QueryTotalDelegationByDelegatorResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUPERFLUID_DELEGATION_RECORDS_FIELD_NUMBER: builtins.int
    DELEGATION_RESPONSE_FIELD_NUMBER: builtins.int
    TOTAL_DELEGATED_COINS_FIELD_NUMBER: builtins.int
    TOTAL_EQUIVALENT_STAKED_AMOUNT_FIELD_NUMBER: builtins.int
    @property
    def superfluid_delegation_records(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[osmosis.superfluid.superfluid_pb2.SuperfluidDelegationRecord]: ...
    @property
    def delegation_response(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.DelegationResponse]: ...
    @property
    def total_delegated_coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def total_equivalent_staked_amount(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(
        self,
        *,
        superfluid_delegation_records: collections.abc.Iterable[osmosis.superfluid.superfluid_pb2.SuperfluidDelegationRecord] | None = ...,
        delegation_response: collections.abc.Iterable[cosmos.staking.v1beta1.staking_pb2.DelegationResponse] | None = ...,
        total_delegated_coins: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
        total_equivalent_staked_amount: cosmos.base.v1beta1.coin_pb2.Coin | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["total_equivalent_staked_amount", b"total_equivalent_staked_amount"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegation_response", b"delegation_response", "superfluid_delegation_records", b"superfluid_delegation_records", "total_delegated_coins", b"total_delegated_coins", "total_equivalent_staked_amount", b"total_equivalent_staked_amount"]) -> None: ...

global___QueryTotalDelegationByDelegatorResponse = QueryTotalDelegationByDelegatorResponse

@typing_extensions.final
class QueryUnpoolWhitelistRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___QueryUnpoolWhitelistRequest = QueryUnpoolWhitelistRequest

@typing_extensions.final
class QueryUnpoolWhitelistResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POOL_IDS_FIELD_NUMBER: builtins.int
    @property
    def pool_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        pool_ids: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_ids", b"pool_ids"]) -> None: ...

global___QueryUnpoolWhitelistResponse = QueryUnpoolWhitelistResponse



from osmosispy.exceptions import QueryError
from osmosis_proto.proto.tendermint.types.block_pb2 import Block
from osmosis_proto.proto.cosmos.tx.v1beta1.tx_pb2 import Tx
from osmosis_proto.proto.cosmos.base.query.v1beta1.pagination_pb2 import PageRequest
from grpc import Channel
from google.protobuf import json_format, message
import grpc._channel
import grpc
from typing import Dict, List, Optional, Union
import collections
import json
import logging
from datetime import datetime
from typing import Any, Callable, Dict, Iterable, List, Optional, Union
from google.protobuf.timestamp_pb2 import Timestamp

# number of decimal places
PRECISION = 18


# reimplementation of cosmos-sdk/types/decimal.go
def to_sdk_dec(dec: float) -> str:
    '''
    create a decimal from an input decimal.
    valid must come in the form:
        (-) whole integers (.) decimal integers
    examples of acceptable input include:
        -123.456
        456.7890
        345
        -456789

    NOTE - An error will return if more decimal places
    are provided in the string than the constant Precision.

    CONTRACT - This function does not mutate the input str.
    '''
    dec_str = str(dec)

    if len(dec_str) == 0:
        raise TypeError(f'Expected decimal string but got: {dec_str}')

    # first extract any negative symbol
    neg = False
    if dec_str[0] == '-':
        neg = True
        dec_str = dec_str[1:]

    if len(dec_str) == 0:
        raise TypeError(f'Expected decimal string but got: {dec_str}')

    strs = dec_str.split('.')
    len_decs = 0
    combined_str = strs[0]

    if len(strs) == 2:  # has a decimal place
        len_decs = len(strs[1])
        if len_decs == 0 or len(combined_str) == 0:
            raise TypeError(f'Expected decimal string but got: {dec_str}')
        combined_str += strs[1]
    elif len(strs) > 2:
        raise TypeError(f'Expected decimal string but got: {dec_str}')

    if len_decs > PRECISION:
        raise TypeError(
            f'value \'{dec_str}\' exceeds max precision by {PRECISION-len_decs} decimal places: max precision {PRECISION}'
        )

    # add some extra zero's to correct to the Precision factor
    zeros_to_add = PRECISION - len_decs
    zeros = '0' * zeros_to_add
    combined_str += zeros

    try:
        int(combined_str, 10)
    except ValueError as err:
        raise ValueError(
            f'failed to set decimal string with base 10: {combined_str}'
        ) from err

    if neg:
        return '-' + combined_str

    return combined_str


def from_sdk_dec_24(dec_str: str) -> float:
    return float(dec_str) * 1e-24


def from_sdk_dec_n(dec_str: str, n: int = 6) -> float:
    return float(dec_str) * 10 ** (-n)


def format_fields_nested(
    object: Union[list, dict], fn: Callable[[Any], Any], fields: List[str]
) -> Union[list, dict]:
    """
    Format the fields inside a nested dictionary with the function provided

    Args:
        object (Union[list, dict]): The object to format
        fn (Callable[[Any], Any]): The function to format objects with
        fields (list[str]): The fields to format

    Returns:
        Union[list, dict]: The output formatted
    """
    if type(object) == dict:
        output = {}
        for k, v in object.items():
            if type(v) in (dict, list):
                output[k] = format_fields_nested(v, fn, fields)
            else:
                if k in fields:
                    output[k] = fn(v)
                else:
                    output[k] = v

        return output

    if type(object) == list:
        output = []

        for element in object:
            if type(object) in (dict, list):
                output.append(format_fields_nested(element, fn, fields))
            else:
                output.append(element)

        return output


def from_sdk_dec(dec_str: str) -> float:
    if dec_str is None or dec_str == '':
        return 0

    if '.' in dec_str:
        raise TypeError(
            f'expected a decimal string but got {dec_str} containing \'.\'')

    try:
        int(dec_str)
    except ValueError as err:
        raise ValueError(f'failed to convert {dec_str} to a number') from err

    neg = False
    if dec_str[0] == '-':
        neg = True
        dec_str = dec_str[1:]

    input_size = len(dec_str)
    bz_str = ''
    # case 1, purely decimal
    if input_size <= PRECISION:
        # 0. prefix
        bz_str = '0.'

        # set relevant digits to 0
        bz_str += '0' * (PRECISION - input_size)

        # set final digits
        bz_str += dec_str
    else:
        # inputSize + 1 to account for the decimal point that is being added
        dec_point_place = input_size - PRECISION

        bz_str = dec_str[:dec_point_place]  # pre-decimal digits
        bz_str += '.'  # decimal point
        bz_str += dec_str[dec_point_place:]  # pre-decimal digits

    if neg:
        bz_str = '-' + bz_str

    return float(bz_str)


def to_sdk_int(i: float) -> str:
    return str(int(i))


def from_sdk_int(int_str: str) -> int:
    return int(int_str)


def toPbTimestamp(dt: datetime):
    ts = Timestamp()
    ts.FromDatetime(dt)
    return ts


class ColoredFormatter(logging.Formatter):
    fmt = "%(asctime)s|%(levelname)s|%(funcName)s| %(message)s"

    white = "\x1b[97;20m"
    grey = "\x1b[38;20m"
    green = "\x1b[32;20m"
    cyan = "\x1b[36;20m"
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"

    FORMATS = {
        logging.DEBUG: fmt.format(green, reset),
        logging.INFO: fmt.format(cyan, reset),
        logging.WARNING: fmt.format(yellow, reset),
        logging.ERROR: fmt.format(red, reset),
        logging.CRITICAL: fmt.format(bold_red, reset),
    }

    def format(self, record: logging.LogRecord):
        """Formats a record for the logging handler.

        Args:
            record (logging.LogRecord): Represents an instance of an event being
                logged.
        """
        log_format = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_format, datefmt="%H:%M:%S")
        return formatter.format(record=record)


def clean_nested_dict(dictionary: Union[List, Dict, str]) -> Dict:
    """
    Takes a nested dictionnary with some values being string json values and convert it into a proper nested
    dictionary.

    Eg ::

        {
            "transaction_fee": "{\"denom\":\"unusd\",\"amount\":\"0\"}",
            "funding_payment": "0.000000000000000000",
            "realized_pnl": "0.000000000000000000",
            "bad_debt": "{\"denom\":\"unusd\",\"amount\":\"0\"}",
            "trader_address": "osmo1zaavvzxez0elundtn32qnk9lkm8kmcsz44g7xl",
            "margin": "{\"denom\":\"unusd\",\"amount\":\"10\"}",
            "exchanged_position_size": "0.004999999999999500",
            "tx_hash": "12E496C996E39820B0807857CB7C19674BDD12DC8D789647D68C50BBB8C7D9CF"
        }

    becomes ::

        {
            "transaction_fee": {
                "denom": "unusd",
                "amount": "0"
            },
            "funding_payment": "0.000000000000000000",
            "realized_pnl": "0.000000000000000000",
            "bad_debt": {
                "denom": "unusd",
                "amount": "0"
            },
            "trader_address": "osmo1zaavvzxez0elundtn32qnk9lkm8kmcsz44g7xl",
            "margin": {
                "denom": "unusd",
                "amount": "10"
            },
            "exchanged_position_size": "0.004999999999999500",
            "tx_hash": "12E496C996E39820B0807857CB7C19674BDD12DC8D789647D68C50BBB8C7D9CF"
        }

    Args:
        dictionary (Union[List, Dict, str]): The dictionary to be converted.

    Returns:
        Dict: A converted dictionary.
    """

    if isinstance(dictionary, str):
        dictionary = json.loads(dictionary)

    for key, value in dictionary.items():
        if isinstance(value, str):
            if value[0] == "{" and value[-1] == "}":
                dictionary[key] = clean_nested_dict(value)
            elif value[0] == "[" and value[-1] == "]":
                try:
                    values_list = json.loads(value)
                    dictionary[key] = [
                        clean_nested_dict(v) for v in values_list]
                except ValueError:
                    dictionary[key] = value
            else:
                dictionary[key] = value

    return dictionary


# ----------------------------------------------------
# ----------------------------------------------------


def dict_keys_must_match(dict_: dict, keys: List[str]):
    """Asserts that two iterables have the same elements, the same number of
    times, without regard to order.
    Alias for the 'element_counts_are_equal' function.

    dict_keys_must_match(dict_, keys)

    Example:
    - [0, 1, 1] and [1, 0, 1] compare equal.
    - [0, 0, 1] and [0, 1] compare unequal.

    """
    assert element_counts_are_equal(dict_.keys(), keys)


def element_counts_are_equal(
    first: Iterable[Any], second: Iterable[Any]
) -> Optional[bool]:
    """Asserts that two iterables have the same elements, the same number of
    times, without regard to order.

    Args:
        first (Iterable[Any])
        second (Iterable[Any])

    Returns:
        Optional[bool]: "passed" status. If this is True, first and second share
            the same element counts. If they don't the function will raise an
            AssertionError and return 'None'.
    """
    first_seq, second_seq = list(first), list(second)

    passed: Union[bool, None]
    try:
        first = collections.Counter(first_seq)
        second = collections.Counter(second_seq)
    except TypeError:
        # Handle case with unhashable elements
        differences = _count_diff_all_purpose(first_seq, second_seq)
    else:
        if first == second:
            passed = True
            return passed
        differences = _count_diff_hashable(first_seq, second_seq)

    if differences:
        standardMsg = "Element counts were not equal:\n"
        lines = ["First has %d, Second has %d:  %r" %
                 diff for diff in differences]
        diffMsg = "\n".join(lines)
        msg = "\n".join([standardMsg, diffMsg])
        passed = False
        assert passed, msg


_Mismatch = collections.namedtuple("Mismatch", "actual expected value")


def _count_diff_all_purpose(actual, expected):
    "Returns list of (cnt_act, cnt_exp, elem) triples where the counts differ"
    # elements need not be hashable
    s, t = list(actual), list(expected)
    m, n = len(s), len(t)
    NULL = object()
    result = []
    for i, elem in enumerate(s):
        if elem is NULL:
            continue
        cnt_s = cnt_t = 0
        for j in range(i, m):
            if s[j] == elem:
                cnt_s += 1
                s[j] = NULL
        for j, other_elem in enumerate(t):
            if other_elem == elem:
                cnt_t += 1
                t[j] = NULL
        if cnt_s != cnt_t:
            diff = _Mismatch(cnt_s, cnt_t, elem)
            result.append(diff)

    for i, elem in enumerate(t):
        if elem is NULL:
            continue
        cnt_t = 0
        for j in range(i, n):
            if t[j] == elem:
                cnt_t += 1
                t[j] = NULL
        diff = _Mismatch(0, cnt_t, elem)
        result.append(diff)
    return result


def _count_diff_hashable(actual, expected):
    "Returns list of (cnt_act, cnt_exp, elem) triples where the counts differ"
    # elements must be hashable
    s, t = collections.Counter(actual), collections.Counter(expected)
    result = []
    for elem, cnt_s in s.items():
        cnt_t = t.get(elem, 0)
        if cnt_s != cnt_t:
            diff = _Mismatch(cnt_s, cnt_t, elem)
            result.append(diff)
    for elem, cnt_t in t.items():
        if elem not in s:
            diff = _Mismatch(0, cnt_t, elem)
            result.append(diff)
    return result


PROTOBUF_MSG_BASE_ATTRS: List[str] = (
    dir(message.Message)
    + ['Extensions', 'FindInitializationErrors', '_CheckCalledFromGeneratedFile']
    + ['_extensions_by_name', '_extensions_by_number']
)
"""PROTOBUF_MSG_BASE_ATTRS (List[str]): The default attributes and methods of
an instance of the 'protobuf.message.Message' class.
"""


def camel_to_snake(camel: str):
    return ''.join(
        ['_' + char.lower() if char.isupper() else char for char in camel]
    ).lstrip('_')


def dict_keys_from_camel_to_snake(d):
    """
    Transform all keys from the dictionary from camelcase to snake case.

    Args:
        d (dict): The dictionary to transform

    Returns:
        dict: The dictionary transformed
    """
    if isinstance(d, list):
        return [
            dict_keys_from_camel_to_snake(
                i) if isinstance(i, (dict, list)) else i
            for i in d
        ]
    return {
        camel_to_snake(a): dict_keys_from_camel_to_snake(b)
        if isinstance(b, (dict, list))
        else b
        for a, b in d.items()
    }


def deserialize(pb_msg: message.Message, no_sdk_transformation: bool = False) -> dict:
    """Deserializes a proto message into a dictionary.

    - sdk.Dec values are converted to floats.
    - sdk.Int values are converted to ints.
    - Missing fields become blank strings.

    Args:
        pb_msg (protobuf.message.Message)
        no_sdk_transformation (bool): Wether to bypass the sdk transformation. Default to False

    Returns:
        dict: 'pb_msg' as a JSON-able dictionary.
    """
    if not isinstance(pb_msg, message.Message):
        return pb_msg
    custom_dtypes: Dict[str, bytes] = {
        str(field[1]): field[0].GetOptions().__getstate__().get("serialized", None)
        for field in pb_msg.ListFields()
    }
    serialized_output = {}
    expected_fields: List[str] = list(pb_msg.DESCRIPTOR.fields_by_name.keys())

    for _, attr in enumerate(expected_fields):
        attr_search = pb_msg.__getattribute__(attr)
        custom_dtype = custom_dtypes.get(str(attr_search))

        if custom_dtype is not None:
            if "sdk/types.Dec" in str(custom_dtype):
                if no_sdk_transformation:
                    serialized_output[str(attr)] = float(
                        pb_msg.__getattribute__(attr))
                else:
                    serialized_output[str(attr)] = utils.from_sdk_dec(
                        pb_msg.__getattribute__(attr)
                    )
            elif "sdk/types.Int" in str(custom_dtype):
                if no_sdk_transformation:
                    serialized_output[str(attr)] = int(
                        pb_msg.__getattribute__(attr))
                else:
                    serialized_output[str(attr)] = utils.from_sdk_int(
                        pb_msg.__getattribute__(attr)
                    )
            elif "Int" in str(custom_dtype):  # Used for sdk.Coin message normalization
                serialized_output[str(attr)] = int(
                    pb_msg.__getattribute__(attr))
            else:
                val = pb_msg.__getattribute__(attr)
                if hasattr(val, '__len__') and not isinstance(val, str):
                    updated_vals = []
                    for v in val:
                        updated_vals.append(deserialize(v))
                    serialized_output[str(attr)] = updated_vals
                else:
                    serialized_output[str(attr)] = deserialize(val)
        elif custom_dtype is None and not attr_search:
            if str(attr_search) == "[]":
                serialized_output[str(attr)] = []
            else:
                serialized_output[str(attr)] = attr_search
        else:
            serialized_output[str(attr)] = deserialize(
                pb_msg.__getattribute__(attr))

    return serialized_output


def deserialize_exp(proto_message: message.Message) -> dict:
    """
    Take a proto message and convert it into a dictionnary.
    sdk.Dec values are converted to be consistent with txs.

    Args:
        proto_message (protobuf.message.Message)

    Returns:
        dict
    """
    output = json_format.MessageToDict(proto_message)

    is_sdk_dec = {
        field.camelcase_name: "types.Dec" in str(field.GetOptions())
        for field in proto_message.DESCRIPTOR.fields
    }

    for field in proto_message.DESCRIPTOR.fields:
        if field.message_type is not None:
            # This is another proto object
            try:
                output[field.camelcase_name] = deserialize_exp(
                    proto_message.__getattribute__(field.camelcase_name)
                )
            except AttributeError:
                output[field.camelcase_name] = output[field.camelcase_name]

        elif is_sdk_dec[field.camelcase_name]:
            output[field.camelcase_name] = utils.from_sdk_dec(
                output[field.camelcase_name]
            )

    return dict_keys_from_camel_to_snake(output)
